/*******************************************************************************
 * 
 * jKiwi -- A virtual makeover and hairstyler application
 * Copyright (c) 2007-2008 Dan Mihai Ile, Maria Joao Roque Barbado Leal
 * 
 * This file is part of jKiwi.
 * 
 * jKiwi is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * jKiwi is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************/

package dialogs;

import java.io.File;
import java.util.ResourceBundle;

import org.eclipse.swt.SWT;
import org.eclipse.swt.SWTException;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

import utils.StockIcon;
import core.Loader;

/**
 * Instances of this class show a few model
 * example images to choose from
 */
public class ExampleImages {

	/**
	 * The shell used by this dialog
	 */
	private Shell sShell = null;

	/**
	 * Composite used to show the buttons to the bottom of the window
	 */
	private Composite compositeBottom = null;

	/**
	 * The close button
	 */
	private Button buttonClose = null;
	
	private final String THUMBNAILS_PATH = "sample-images/thumbnails";
	private final String IMAGES_PATH = "sample-images/large";
	
	private String selectedImage = null;

	private Label labelBefore = null;

	/**
	 * This dialog's Shell parent
	 */
	private final Shell parent;

	/**
	 * messages that are to be used by the entire GUI
	 */
	private ResourceBundle messages;

	//TODO check for the exceptions in the comment
	/**
	 * Creates a new instance of this class given it's parent Shell window
	 * @param parent the dialog's parent
	 *
	 * @exception IllegalArgumentException <ul>
	 *    <li>ERROR_NULL_ARGUMENT - if the parent is null</li>
	 * </ul>
	 * @exception SWTException <ul>
	 *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the parent</li>
	 *    <li>ERROR_INVALID_SUBCLASS - if this class is not an allowed subclass</li>
	 * </ul>
	 */
	public ExampleImages(Shell parent) {
		this.parent = parent;
		messages = Loader.getMessages();
		createSShell();
	}

	/**
	 * This method initializes sShell
	 */
	private void createSShell() {

		sShell = new Shell(parent, SWT.APPLICATION_MODAL | SWT.DIALOG_TRIM);
		sShell.setText(messages.getString("EXAMPLE_IMAGES_WINDOW_TITLE"));
		sShell.setLayout(new GridLayout());

		labelBefore = new Label(sShell, SWT.NONE);
		labelBefore.setText(messages.getString("EXAMPLE_IMAGES_WINDOW_LABEL"));

		createCompositeButtons();
		createCompositeBottom();

		//dispose the two created images on disposing this dialog
		sShell.addDisposeListener(new DisposeListener() {
			public void widgetDisposed(DisposeEvent e) {
			}
		});
	}
	
	public String open() {
		sShell.pack();
		sShell.open();
		while (!sShell.isDisposed()) {
			if (!sShell.getDisplay().readAndDispatch())
				sShell.getDisplay().sleep();
		}
		return selectedImage;
	}

	/**
	 * This method initializes compositeExamples
	 */
	private void createCompositeButtons() {
		Composite compositeExamples = new Composite(sShell, SWT.BORDER);
		compositeExamples.setLayout(new RowLayout());
		compositeExamples.setLayoutData(new GridData());
		
		File thumbsDir = new File(System.getProperty("program.resourcespath") + File.separator + THUMBNAILS_PATH + File.separator);
		File imagesDir = new File(System.getProperty("program.resourcespath") + File.separator + IMAGES_PATH + File.separator);
		
		for (File f : thumbsDir.listFiles()) {
			final File largeImgFile = new File(imagesDir.getAbsolutePath() + File.separator + f.getName());
			if (f.isFile() && largeImgFile.exists()) {
				try {
					Button b1 = new Button(compositeExamples, SWT.NONE);
					b1.setImage(new Image(Display.getCurrent(), new ImageData(f.getAbsolutePath())));
					b1.addSelectionListener(new org.eclipse.swt.events.SelectionAdapter() {
						public void widgetSelected(org.eclipse.swt.events.SelectionEvent e) {
							selectedImage = largeImgFile.getAbsolutePath();
							sShell.dispose();
						}
					});
				} catch (Exception e) {
					// skip the file
				}
			}
		}
	}

	/**
	 * This method initializes compositeBottom
	 */
	private void createCompositeBottom() {
		GridData gridDataButtonClose = new GridData();
		GridLayout gridLayoutComposite = new GridLayout();
		gridLayoutComposite.marginHeight = 1;
		gridLayoutComposite.marginWidth = 1;
		GridData gridDataComposite = new GridData();
		gridDataComposite.horizontalSpan = 2;
		gridDataComposite.horizontalAlignment = org.eclipse.swt.layout.GridData.END;
		compositeBottom = new Composite(sShell, SWT.NONE);
		compositeBottom.setLayoutData(gridDataComposite);
		compositeBottom.setLayout(gridLayoutComposite);
		buttonClose = new Button(compositeBottom, SWT.NONE);
		buttonClose.setImage(new Image(Display.getCurrent(), StockIcon.getIcon(StockIcon.ICON_CLOSE, StockIcon.SIZE_LARGE_TOOLBAR, true)));
		buttonClose.setLayoutData(gridDataButtonClose);
		buttonClose.setText(messages.getString("EXAMPLE_IMAGES_WINDOW_CLOSE_BUTTON"));
		buttonClose.addSelectionListener(new org.eclipse.swt.events.SelectionAdapter() {
			public void widgetSelected(org.eclipse.swt.events.SelectionEvent e) {
				sShell.close();
			}
		});
		gridDataButtonClose.widthHint = buttonClose.computeSize(-1, -1).x + 10;
	}
}